#!/bin/bash
#MSUB -r WeightsMask        # Job name
#MSUB -o Out_WeightsMask    # Standard output
#MSUB -e Out_WeightsMask    # Error output
#MSUB -n 1                  # Number of processors
#MSUB -T 7200              # Time limit (seconds)
#MSUB -Q normal
#MSUB -q skylake
#MSUB -p devcmip6
#MSUB -m work,store,scratch

### ===========================================================================
###
### Creates interpolation weights between ORCA and atmosphere grids.
### Interpolates ORCA mask to atmosphere grid.
### Weight files are at OASIS-MCT format.
###
### Atmosphere grid may be lon/lat LMDZ or DYNAMICO icosahedron
###
### Documentation : https://forge.ipsl.jussieu.fr/igcmg/wiki/IPSLCM6/MOSAIX
### ===========================================================================
##
##  MOSAIX is under CeCILL_V2 licence. See "Licence_CeCILL_V2-en.txt"
##  file for an english version of the licence and
##  "Licence_CeCILL_V2-fr.txt" for a french version.
##
##  Permission is hereby granted, free of charge, to any person or
##  organization obtaining a copy of the software and accompanying
##  documentation covered by this license (the "Software") to use,
##  reproduce, display, distribute, execute, and transmit the
##  Software, and to prepare derivative works of the Software, and to
##  permit third-parties to whom the Software is furnished to do so,
##  all subject to the following:
##
##  Warning, to install, configure, run, use any of MOSAIX software or
##  to read the associated documentation you'll need at least one (1)
##  brain in a reasonably working order. Lack of this implement will
##  void any warranties (either express or implied).  Authors assumes
##  no responsability for errors, omissions, data loss, or any other
##  consequences caused directly or indirectly by the usage of his
##  software by incorrectly or partially configured
##
##
## SVN information
#  $Author$
#  $Date$
#  $Revision$
#  $Id$
#  $HeadURL$
#
## Tested with : 
# CplModel=eORCA1.2xLMD144142 ; qsub -r ${CplModel} -o Out_${CplModel} -e Out_${CplModel} CreateWeightsMask.bash
# CplModel=ORCA2.3xLMD9695
# CplModel=ORCA2.3xICO30
# CplModel=ORCA2.3xICO40
# CplModel=eORCA1.2xICO40
# CplModel=eORCA1.2xICO450
# CplModel=eORCA1.2xLMD256256
# CplModel=eORCA025.1xLMD144142
# CplModel=eORCA025.1xLMD256256
#

set +vx
# Prefixes for pretty print in color
export Bold=$(tput bold)  
export Unde=$(tput smul) ; export OffUnde=$(tput rmul)
export Stou=$(tput smso) ; export OffStou=$(tput rmso)
export Reve=$(tput rev ) 
couleurs=( "Black" "Blue" "Green" "Cyan" "Red" "Magenta" "Yellow" "White" )
for i in $(seq 0 7) ; do eval "export ${couleurs[$i]}=$(tput setf ${i})" ; done
export Norm=$(tput sgr0)
export Titre=${Bold}${Blue}

##
## Configuration
## ===========================================================================
set -e
echo BASH Version ${BASH_VERSION}
echo SHELL ${SHELL}

#
echo ${Titre}"Defines model"${Norm}
# =================================
CplModel=ORCA2.3xLMD9695
#CplModel=ORCA2.3xICO30
#CplModel=ORCA2.3xICO40
#CplModel=eORCA1.2xLMD144142
#CplModel=eORCA1.4.0xLMD144142
#CplModel=eORCA1.4.2xLMD144142
#CplModel=eORCA1.2xLMD256256
#CplModel=eORCA1.2xICO40
#CplModel=eORCA1.4.2xICO40
#CplModel=eORCA1.2xICO450
#CplModel=eORCA025.1xLMD256256

#Version="v0" ; Comment="Fully tested in IPSLCM6 eORCA1.2 x LMD 144x142"
#Version="v1" ; Comment="Fully tested in IPSLCM6 eORCA1.2 x LMD 144x142"
#Version="NoSearchRadius" ; Comment="For testing new routing"
Version="v2" ; Comment="Correction of ORCA masks to have a perfect conservation of run-off"

# If available, get model name from job name
if [[ X${SLURM_JOB_NAME} = X*ORC* ||  X${SLURM_JOB_NAME} = X*LMD*  ||  X${SLURM_JOB_NAME} = X*ICO* ]] ; then
    CplModel=${SLURM_JOB_NAME} ; 
fi
OCE=${CplModel//x*} ; ATM=${CplModel##*x}

ModelName="IPSL CM6"
[[ ${ATM} = ICO*      ]] && ModelName="IPSL CM7"

echo ${Titre}"ATM model  : ${ATM}"      ${Norm}
echo ${Titre}"OCE model  : ${OCE}"      ${Norm}
echo ${Titre}"Cpl model  : ${CplModel}" ${Norm}
echo ${Titre}"Version    : ${Version}"  ${Norm}
echo ${Titre}"Model Name : ${ModelName}"${Norm}

# Runoff parameter. atmCoastWidth and oceCoastWidth in grid points, searchRadius in km
atmCoastWidth=2 ; oceCoastWidth=2 ; searchRadius=550.0
runOff_atmQuantity=Quantity ; runOff_oceQuantity=Surfacic

# Specific cases
[[ ${ATM} = ICO*      ]] && atmCoastWidth=0 # Parameter relevant for LMD rectilinear grid only
[[ ${ATM} = LMD*      ]] && atmCoastWidth=2
[[ ${OCE} = ORCA2.*   ]] && oceCoastWidth=1
[[ ${OCE} = eORCA1.*  ]] && oceCoastWidth=2
[[ ${OCE} = eORCA025* ]] && oceCoastWidth=1

# More specific cases
[[ ${CplModel} = eORCA1.*xLMD144142 ]] && { atmCoastWidth=2 ; oceCoastWidth=2 ; searchRadius=550.0 ; }

if [[ ${CplModel} = eORCA1.*xLMD144142 && Version = "NoSearchRadius" ]] ; then
    atmCoastWidth=2 ; oceCoastWidth=2 ; searchRadius=0.0
fi

# Default values, used to create ocean fraction on atmospheric grid 
DefaultValues=( Direction=o2a,oceGrid=t,atmGrid=t,Order=1st,Quantity=false,Renormalize=false,useArea=false,maskSrc=true,maskDst=false,Name=OceFrac )

## List of weights to build
## ====================================================================================================================================
#
# Each item in CommandList describes the properties of interpolation weights to generate. 
# White spaces separate analysis.
#
# \!/ No spaces in any analysis \!/
#
# Specific commands : 'Runoff', 'Calving'
#
# Keywords : 
#   Direction   : o2a for ocean to atmosphere, a2o for atmosphere to ocean.
#   Order       : 1st or 2nd.
#   Quantity    : true if integrated quantity over a grid box (i.e runoff on atmosphere side),
#                 false for flux (quantity / m^2) or intensive value (temperature, salinity, sea-ice fraction, ...).
#   Renormalize : used when source grid is masked, to use values on non masked points only.
#   oceGrid     : t, u or v point for NEMO C grid.
#   atmGrid     : up to know, only t grid used in the atmosphere.
#   useArea     : if true area from the model metrics is used. If false, areas are computed by XIOS from grid corners.
#   maskSrc     : true to use the source grid mask, false to used all grid points.
#   maskDst     : true to use the destination grid mask, false to use all grid points.
#
## Classic cases for IPSLCM6
AtmOceFluxes="   Direction=a2o,Order=1st,Quantity=false,Renormalize=false,atmGrid=t,oceGrid=t,useArea=true,maskSrc=true,maskDst=true,Name=HeatWaterFluxes" # Heat and water fluxes
OceAtmTemp="     Direction=o2a,Order=1st,Quantity=false,Renormalize=true,oceGrid=t,atmGrid=t,useArea=false,maskSrc=true,maskDst=true,Name=TempIceAlb"      # Temperature, sea-ice fraction, albedo
OceAtmTemp2nd="  Direction=o2a,Order=2nd,Quantity=false,Renormalize=true,oceGrid=t,atmGrid=t,useArea=false,maskSrc=true,maskDst=true,Name=TempIceAlb"      # Temperature, sea-ice fraction, albedo
AtmOceStressU="  Direction=a2o,Order=2nd,Quantity=false,Renormalize=false,atmGrid=t,oceGrid=u,useArea=true,maskSrc=true,maskDst=true,Name=WindStress"      # Wind stress to NEMO U point
AtmOceStressV="  Direction=a2o,Order=2nd,Quantity=false,Renormalize=false,atmGrid=t,oceGrid=v,useArea=true,maskSrc=true,maskDst=true,Name=WindStress"      # Wind stress to NEMO V point
AtmOceQuantity=" Direction=a2o,Order=1st,Quantity=true,Renormalize=false,atmGrid=t,oceGrid=t,useArea=false,maskSrc=true,maskDst=true,Name=Quantity"        # e.g. runoff
# For new parameterization
AtmOceTemp="     Direction=o2a,Order=1st,Quantity=false,Renormalize=true,oceGrid=t,atmGrid=t,useArea=false,maskSrc=true,maskDst=true,Name=OceTemp" # e.g. T and S correction from DWL parametrization
OceAtmFluxes="   Direction=o2a,Order=1st,Quantity=false,Renormalize=false,atmGrid=t,oceGrid=t,useArea=true,maskSrc=true,maskDst=true,Name=OceAtmFluxes" # e.g. CO2 fluxes
# Test to do with 2nd order
AtmOceFluxes2nd="Direction=a2o,Order=2nd,Quantity=false,Renormalize=false,atmGrid=t,oceGrid=t,useArea=true,maskSrc=true,maskDst=true,Name=HeatWaterFluxes" # Heat and water fluxes
OceAtmFluxes2nd="Direction=o2a,Order=2nd,Quantity=false,Renormalize=false,atmGrid=t,oceGrid=t,useArea=true,maskSrc=true,maskDst=true,Name=OceAtmFluxes" # e.g. CO2 fluxes
AtmOceTemp2nd="  Direction=o2a,Order=2nd,Quantity=false,Renormalize=true,oceGrid=t,atmGrid=t,useArea=false,maskSrc=true,maskDst=true,Name=OceTemp" # e.g. T and S correction from DWL parametrization

## Creates the list

# Standard list for IPSLCM6
#CommandList=( ${AtmOceFluxes} ${OceAtmTemp} ${AtmOceStressU} ${AtmOceStressV} ${OceAtmFluxes} Runoff Calving Grids )

# More comprehensive list for IPSLCM6 with new features
#CommandList=( ${AtmOceFluxes} ${OceAtmTemp} ${AtmOceStressU} ${AtmOceStressV} ${OceAtmFluxes} ${AtmOceTemp} Runoff Calving Grids )

# With 2nd order
CommandList=( ${AtmOceFluxes} ${AtmOceFluxes2nd} ${OceAtmTemp} ${OceAtmTemp2nd} ${AtmOceStressU} ${AtmOceStressV} ${OceAtmFluxes} ${OceAtmFluxes2nd} ${AtmOceTemp} ${AtmOceTemp2nd} Runoff Calving Grids )

# Debugs
#CommandList=( ${AtmOceFluxes} ${OceAtmTemp} Runoff Calving Grids )
#CommandList=( ${AtmOceFluxes} Runoff Calving Grids )
#CommandList=( ${AtmOceFluxes} ${OceAtmTemp} )
#CommandList=( ${AtmOceFluxes} )
#CommandList=( Runoff )
#CommandList=( Runoff Calving Grids )
#CommandList=( Calving )
#CommandList=( Grids )

## ===========================================================================
##
## You should not change anything below this line ....
##
## ===========================================================================
#Tag="MOSAIX"
SUBMIT_DIR=$(pwd)
FMT_XIOS=netcdf4

# Functions to handle command parameters
# ======================================
function tolow { echo ${*} | tr '[:upper:]' '[:lower:]' ; }
function toup  { echo ${*} | tr '[:lower:]' '[:upper:]' ; }
function capit { echo $(toup ${1:0:1})$(tolow ${1:1})   ; }
function Capit { echo $(toup ${1:0:1})${1:1}            ; }

function readCommand {
    # Decipher the command line to set bash variables 
    local l_Debug="no" l_Element
    while [[ ${1} = -* ]] ; do
	case ${1} in
	    ( -- ) shift ; break ;;
	    ( -d | --debug ) l_Debug="true" ; shift ;;
	esac
    done
    local l_Command=${1}
    for l_Element in $(echo ${l_Command} | tr "," "\n" ) ; do
	[[ "X${l_Debug}" = "Xtrue" ]] && echo ${l_Element}
	eval export ${l_Element}
    done
}

function setValues {
    ## Read a command line to define shell variables
    # Set default values
    readCommand "Direction=None,Order=None,Quantity=None,Renormalize=None,atmGrid=None,oceGrid=None,useArea=None,maskSrc=None,maskDst=None"
    # Read command line
    readCommand ${1}
    #
    #oceGrid=${oceGrid,,} ; atmGrid=${atmGrid,,} # Model names to lower case
    #OCEGRID=${oceGrid^^} ; ATMGRID=${atmGrid^^} # Model names to upper case
    oceGrid=$(tolow ${oceGrid}) ; atmGrid=$(tolow ${atmGrid})
    OCEGRID=$(toup  ${oceGrid}) ; ATMGRID=$(toup  ${atmGrid})

    case ${Order} in
	( 1st ) numOrder=1 ;;
	( 2nd ) numOrder=2 ;;
    esac
    case ${Renormalize} in
	( true )  NormName=Normalized   ;;
	( false ) NormName=UnNormalized ;;
    esac
    case ${Quantity} in
	( true )  QuantName=Integrated ;;
	( false ) QuantName=Surfacic   ;;
    esac
    case ${useArea} in
	( true )  AreaName=Area   ;;
	( false ) AreaName=NoArea ;;
    esac

    if [[ "${Name}" != "None" ]] ; then
	FullName=${Name}_${Order}Order
    else
	FullName=${Order}Order_${NormName}_${QuantName}_${AreaName}
    fi

    case ${Direction} in
	( o2a )
	src=${oce} ; SRC=${OCE} ; srcGrid=${oceGrid} ; srcDomainType=${oceDomainType} ; SRCGRID=${OCEGRID} ; srcArea=area_grid_${OCEGRID}
	dst=${atm} ; DST=${ATM} ; dstGrid=${atmGrid} ; dstDomainType=${atmDomainType} ; DSTGRID=${ATMGRID} ; dstArea=aire
	;; 
	( a2o )
	src=${atm} ; SRC=${ATM} ; srcGrid=${atmGrid} ; srcDomainType=${atmDomainType} ; SRCGRID=${ATMGRID} ; srcArea=aire
	dst=${oce} ; DST=${OCE} ; dstGrid=${oceGrid} ; dstDomainType=${oceDomainType} ; DSTGRID=${OCEGRID} ; dstArea=area_grid_${OCEGRID}
	;;
    esac
    echo ${Green}"${SRC} ${SRCGRID} toward ${DST} ${DSTGRID} - ${Order} Order - Normalize: ${Renormalize} - Quantity: ${QuantName} - Area: ${AreaName}  "${Norm}
    echo ${Green}"FullName : ${FullName}"${Norm}
}

#
# Defines computer
# ================
if [[ $(hostname) = irene*    ]] ; then arch=irene ; center=tgcc ; fi
if [[ $(hostname) = lsce*     ]] ; then arch=spip  ; center=spip ; fi

case ${arch} in
    ( irene )
    set +vx
    set +e
    R_IN=$(ccc_home -u igcmg --cccwork)/IGCM
    TMPDIR=${CCCWORKDIR}/TMP
    SUBMIT_DIR=${BRIDGE_MSUB_PWD:-${SUBMIT_DIR}}
    MpiRun="time ccc_mprun"
    PyRun="time ccc_mprun -n 1" # Needed to force python to run on one process only
    module purge
    source ${SUBMIT_DIR}/arch.env
    module load nco/4.9.1
    module load cdo/1.9.5
    module load python3 
    module load datadir/igcmg
    module list
    cp ${SUBMIT_DIR}/arch.env . ${TMPDIR}
    set -e
    ;;
    ( spip )
    R_IN=${HOME}/Scratch/IGCM
    TMPDIR=${HOME}/Scratch/TMP
    SUBMIT_DIR=$(pwd)
    MpiRun="/opt/local/bin/mpirun -n 1"
    PyRun="time"
    ;; 
    ( * ) exit -1 ;;
esac

set -x ; set -e

mkdir -p ${TMPDIR}/${CplModel} || exit 1
cd       ${TMPDIR}/${CplModel} || exit 2
rm -fr *

#
# Suffixes
# ---------------------------------------------------------------------------

case ${OCE} in
    ( *ORC* )         oce=orc ; oceDomainType=curvilinear   ;;
esac

case ${ATM} in
    ( *dynamico*    ) atm=ico ; atmDomainType=unstructured  ;;
    ( *ICO*         ) atm=ico ; atmDomainType=unstructured  ;;
    ( *lmd* | *LMD* ) atm=lmd ; atmDomainType=rectilinear   ;;
esac

case ${OCE} in # Periodicity type of ORCA grid
    ( ORCA2.4              ) OcePerio=4.2 ;; # 4 ORCA 6 PALEORCA
    ( ORCA2*               ) OcePerio=4   ;; # 4 ORCA 6 PALEORCA
    ( eORCA1.2             ) OcePerio=6   ;;
    ( eORCA1.4.0           ) OcePerio=6   ;;
    ( eORCA1.4.2           ) OcePerio=6.2 ;;
    ( ORCA1*               ) OcePerio=6   ;;
    ( eORCA025.4           ) OcePerio=6.2 ;;
    ( ORCA025* | eORCA025* ) OcePerio=6   ;;
esac
#

cp ${SUBMIT_DIR}/bin/interpol.exe       .
cp ${SUBMIT_DIR}/*.py                   .
cp ${SUBMIT_DIR}/iodef_atm_to_oce.xml   .
cp ${SUBMIT_DIR}/iodef_oce_to_atm.xml   .

[[ -f ${R_IN}/OCE/NEMO/${OCE}/${OCE}_coordinates_mask.nc       ]] && cp ${R_IN}/OCE/NEMO/${OCE}/${OCE}_coordinates_mask.nc        .
[[ -f ${R_IN}/OCE/NEMO/${OCE}/GRIDS/${OCE}_coordinates_mask.nc ]] && cp ${R_IN}/OCE/NEMO/${OCE}/GRIDS/${OCE}_coordinates_mask.nc  .
if [[ ! -f ${OCE}_coordinates_mask.nc ]] ; then
    exit 1
fi

cp ${R_IN}/ATM/GRID/${ATM}_grid.nc .

# Soucis a corriger plus tard
if [[ ${ATM} = LMD9695 ]] ; then
    cat <<EOF > toDouble.nco
lat                = double (lat) ;
lon                = double (lon) ;
aire               = double (aire) ;
fract_oce          = double (fract_oce) ;
fract_sic          = double (fract_sic) ;
fract_oce_plus_sic = double (fract_oce_plus_sic) ;
EOF
    ncap2 --history --overwrite --script-file toDouble.nco ${ATM}_grid.nc tmp_${ATM}_grid.nc
    mv tmp_${ATM}_grid.nc ${ATM}_grid.nc
fi

##
echo ${Titre}"NEMO T point towards ATM - 1st order"${Norm}
## ===========================================================================
echo "Command parameters : ${Command}"
setValues ${DefaultValues}

OutFileName=${srcGrid}${src}_to_${dstGrid}${dst}_${FullName}

cp iodef_oce_to_atm.xml iodef.xml

cat << EOF > command_file_${OutFileName}.txt
-n 'context[@id="interpol_read"]/file_definition/file[@id="file_src"]/field[@id="mask_src"]'     --key name=maskutil_T
-n 'context[@id="interpol_read"]/file_definition/file[@id="file_src"]/field[@id="area_src"]'     --key name=area_grid_T
-n 'context[@id="interpol_read"]/domain_definition/domain[@id="domain_src"]'                     --key type=${srcDomainType}
-n 'context[@id="interpol_read"]/domain_definition/domain[@id="domain_dst"]'                     --key type=${dstDomainType}
-n 'context[@id="interpol_run"]/domain_definition/domain[@id="domain_dst"]/interpolate_domain'   --key order=1
-n 'context[@id="interpol_run"]/domain_definition/domain[@id="domain_dst"]/interpolate_domain'   --key quantity=false
-n 'context[@id="interpol_run"]/domain_definition/domain[@id="domain_dst"]/interpolate_domain'   --key renormalize=false
-n 'context[@id="interpol_run"]/domain_definition/domain[@id="domain_dst"]/interpolate_domain'   --key use_area=false
-n 'context[@id="interpol_run"]/file_definition/file[@id="file_src"]/field[@id="mask_source"]'   --key name=maskutil_T
-n 'context[@id="interpol_run"]/file_definition/file[@id="file_src"]/field[@id="area_source"]'   --key name=area_grid_T
-n 'context[@id="interpol_run"]/file_definition/file[@id="dia"]'                                 --key name=dia_${OutFileName}
-n 'context[@id="interpol_run"]/file_definition/file[@id="dia"]/variable[@name="title"]'         --text "${SRC} mask interpolated to ${DST}"
-n 'context[@id="interpol_run"]/file_definition/file[@id="dia"]/variable[@name="source_grid"]'   --text ${srcDomainType}
-n 'context[@id="interpol_run"]/file_definition/file[@id="dia"]/variable[@name="dest_grid"]'     --text ${dstDomainType}
-n 'context[@id="interpol_run"]/file_definition/file[@id="dia"]/variable[@name="order"]'         --text 1
-n 'context[@id="interpol_run"]/domain_definition/domain[@id="domain_src"]'                      --key type=${srcDomainType}
-n 'context[@id="interpol_run"]/domain_definition/domain[@id="domain_dst"]'                      --key type=${dstDomainType}
-n 'context[@id="interpol_run"]/domain_definition/domain[@id="domain_dst"]/interpolate_domain'   --key weight_filename=rmp_${OutFileName}.nc
-n 'context[@id="interpol_run"]/domain_definition/domain[@id="domain_dst"]/interpolate_domain'   --key order=1
-n 'context[@id="interpol_run"]/file_definition/file[@id="dia"]/variable[@name="normalization"]' --text false
-n 'context[@id="interpol_run"]/file_definition/file[@id="dia"]/variable[@name="quantity"]'      --text false
-n 'context[@id="interpol_run"]/file_definition/file[@id="dia"]/variable[@name="use_area"]'      --text false
-n 'context[@id="interpol_run"]/domain_definition/domain[@id="domain_dst"]/interpolate_domain'   --key renormalize=false
-n 'context[@id="interpol_run"]/domain_definition/domain[@id="domain_dst"]/interpolate_domain'   --key quantity=false
-n 'context[@id="interpol_run"]/domain_definition/domain[@id="domain_dst"]/interpolate_domain'   --key use_area=false
-n 'context[@id="interpol_run"]/file_definition/file[@id="dia"]/variable[@name="Model"]'         --text "${ModelName}"
EOF

python3 update_xml.py -i iodef.xml -c command_file_${OutFileName}.txt
cp iodef.xml iodef_${OutFileName}.xml

ln -fs ${OCE}_coordinates_mask.nc  oce_grid.nc
ln -fs ${ATM}_grid.nc              atm_grid.nc

${MpiRun} ./interpol.exe --mask_src=${maskSrc} --mask_dst=${maskDst} --use_area=${useArea}

for File in rmp_${OutFileName}.nc dia_${OutFileName}.nc ; do
    ncatted --history \
	    --attribute map_method,global,o,c,"Conservative Remapping - 1st order"  \
	    --attribute normalization,global,o,c,"false"  \
	    --attribute Quantity,global,o,c,"false"  \
	    --attribute UseArea,global,o,c,"false"  rmp_${OutFileName}.nc 
done
##
echo ${Titre}"Correct spurious values (extremes)"${Norm}
## ===========================================================================
cat <<EOF > correction_masque.nco
where (OceFrac < 0.00001) OceFrac=OceFrac.get_miss() ;
where (OceFrac > 0.99999) OceFrac=1.0 ;
OceFrac.delete_miss() ;
// Fill masked values to land values
where (OceFrac >  1.0) OceFrac=0.0 ;
where (OceFrac <  0.0) OceFrac=0.0 ; 
OceMask = OceFrac ;
EOF
ncap2 --history --overwrite --script-file correction_masque.nco dia_t${oce}_to_t${atm}_${FullName}.nc tmp_dia_t${oce}_to_t${atm}_${FullName}.nc ; mv tmp_dia_t${oce}_to_t${atm}_${FullName}.nc dia_t${oce}_to_t${atm}_${FullName}.nc
ncatted --history -a missing_value,OceFrac,d,,"" -a _FillValue,OceFrac,d,,"" -a missing_value,OceMask,d,,"" -a _FillValue,OceMask,d,,"" dia_t${oce}_to_t${atm}_${FullName}.nc
 
##
echo ${Titre}"Creates mask on ATM grid"${Norm}
## ===========================================================================
cp dia_t${oce}_to_t${atm}_${FullName}.nc  dia_t${oce}_to_t${atm}_${FullName}_mask.nc
ncks --alphabetize --history --overwrite --variable OceFrac dia_t${oce}_to_t${atm}_${FullName}_mask.nc ${ATM}_grid_maskFrom_${OCE}.nc
ncatted  --history --attribute name,global,m,c,"${ATM}_grid_maskFrom_${OCE}.nc" ${ATM}_grid_maskFrom_${OCE}.nc

cat <<EOF > creation_masque.nco
Oce2AtmMask = OceMask ; 
where (OceMask >  0.0) Oce2AtmMask=1 ;
where (OceMask <= 0.0) Oce2AtmMask=0 ;
EOF

ncap2 --history --overwrite --script-file creation_masque.nco dia_t${oce}_to_t${atm}_${FullName}.nc tmp_dia_${ATM}_grid_maskFrom_${OCE}.nc 
ncks --overwrite --history --variable OceMask,OceFrac,Oce2AtmMask tmp_dia_${ATM}_grid_maskFrom_${OCE}.nc ${ATM}_grid_maskFrom_${OCE}.nc

if [[ $(ncdump -h ${ATM}_grid_maskFrom_${OCE}.nc | grep domain_dst | wc -l) -gt 0 ]] ; then
    echo "Change dimension names, and some attributes accordingly"
    ## ===========================================================
    case ${atm} in
	( *ico* ) 
	ncrename --history \
		 --dimension cell_domain_dst,cell \
		 --dimension nvertex_domain_dst,nvertex \
		 --variable lat_domain_dst,lat               --variable lon_domain_dst,lon  \
		 --variable bounds_lat_domain_dst,bounds_lat --variable bounds_lon_domain_dst,bounds_lon ${ATM}_grid_maskFrom_${OCE}.nc
	ncatted  --history --attribute bounds,lat,m,c,"bounds_lat" --attribute bounds,lon,m,c,"bounds_lon" ${ATM}_grid_maskFrom_${OCE}.nc
	;;
	( *lmd* )
	ncrename --history \
		 --variable  lon_domain_dst,lon --variable  lat_domain_dst,lat \
		 --dimension lon_domain_dst,lon --dimension lat_domain_dst,lat ${ATM}_grid_maskFrom_${OCE}.nc
	;;
    esac
    
    ncatted  --history \
	     --attribute coordinates,OceFrac,m,c,"lat lon"\
	     --attribute coordinates,OceMask,m,c,"lat lon" \
	     --attribute coordinates,Oce2AtmMask,m,c,"lat lon" \
	     --attribute long_name,OceFrac,c,c,"fraction of ocean in a grid cell" \
	     --attribute units,OceFrac,c,c,"[0,1]" \
	     --attribute long_name,OceMask,c,c,"fraction of ocean in a grid cell" \
	     --attribute units,OceMask,c,c,"[0,1]" \
	     --attribute long_name,Oce2AtmMask,c,c,"land mask" \
	     --attribute units,Oce2AtmMask,c,c,"Land:0, Ocean:1" ${ATM}_grid_maskFrom_${OCE}.nc
fi

ncks --history --alphabetize --append --variable aire atm_grid.nc ${ATM}_grid_maskFrom_${OCE}.nc
ncatted  --history --attribute units,aire,m,c,"m^2" ${ATM}_grid_maskFrom_${OCE}.nc

[[ ${atm} = *ico* ]] && ncks --alphabetize --history --append --variable bounds_lon,bounds_lat atm_grid.nc ${ATM}_grid_maskFrom_${OCE}.nc

##
#echo ${Titre}"Creates mask of coastal OCE points"${Norm}
## ===========================================================================
#${PyRun} python3 -u ComputeNemoCoast.py -n ${OcePerio} -i ${OCE}_coordinates_mask.nc # Creates OceCoastal

##
#echo ${Titre}"Creates mask of coastal ATM points"${Norm}
## ===========================================================================
#cat <<EOF > coastal.nco
#AtmCoastal = OceFrac * 0.0 ;
#where (OceFrac > 0.0 && OceFrac < 1.0 )  AtmCoastal = 1.0 ;
#EOF
#ncap2 --history --overwrite --script-file coastal.nco ${ATM}_grid_maskFrom_${OCE}.nc ${ATM}_coastal_maskFrom_${OCE}.nc
#ncks --history --append --variable AtmCoastal ${ATM}_coastal_maskFrom_${OCE}.nc ${ATM}_grid_maskFrom_${OCE}.nc
#rm ${ATM}_coastal_maskFrom_${OCE}.nc

##
echo ${Titre}"Other weights files"${Norm}
## ===========================================================================
# Loop on commands
for Command in ${CommandList[@]}
do
    echo "Command parameters : ${Command}"
    setValues ${Command}
    if [[ ${Command} = "Runoff"  ]] ; then okRunoff=yes  ; continue ; fi
    if [[ ${Command} = "Calving" ]] ; then okCalving=yes ; continue ; fi
    if [[ ${Command} = "Grids"   ]] ; then okGrids=yes   ; continue ; fi
    
    ln -fs ${OCE}_coordinates_mask.nc     oce_grid.nc
    ln -fs ${ATM}_grid_maskFrom_${OCE}.nc atm_grid.nc

    OutFileName=${srcGrid}${src}_to_${dstGrid}${dst}_${FullName}
    
    case ${Direction} in
	( o2a )
	cp iodef_oce_to_atm.xml iodef.xml
	cat << EOF > command_file_${OutFileName}.txt
-n 'context[@id="interpol_read"]/file_definition/file[@id="file_src"]/field[@id="mask_src"]'   --key name=maskutil_${DSTGRID} 
-n 'context[@id="interpol_read"]/file_definition/file[@id="file_dst"]/field[@id="mask_dst"]'   --key name=Oce2AtmMask
-n 'context[@id="interpol_run"]/file_definition/file[@id="file_src"]/field[@id="mask_source"]' --key name=maskutil_${SRCGRID}
-n 'context[@id="interpol_run"]/file_definition/file[@id="file_src"]/field[@id="area_source"]' --key name=area_grid_${SRCGRID}
-n 'context[@id="interpol_run"]/file_definition/file[@id="file_dst"]/field[@id="mask_dest"]'   --key name=Oce2AtmMask
-n 'context[@id="interpol_run"]/file_definition/file[@id="file_dst"]/field[@id="area_dest"]'   --key name=aire
EOF
	;;
	( a2o )
	cp iodef_atm_to_oce.xml iodef.xml
	cat << EOF > command_file_${OutFileName}.txt
-n 'context[@id="interpol_read"]/file_definition/file[@id="file_src"]/field[@id="mask_src"]'   --key name=Oce2AtmMask
-n 'context[@id="interpol_read"]/file_definition/file[@id="file_dst"]/field[@id="mask_dst"]'   --key name=mask_${DSTGRID}
-n 'context[@id="interpol_run"]/file_definition/file[@id="file_src"]/field[@id="mask_source"]' --key name=Oce2AtmMask
-n 'context[@id="interpol_run"]/file_definition/file[@id="file_src"]/field[@id="area_source"]' --key name=aire
-n 'context[@id="interpol_run"]/file_definition/file[@id="file_dst"]/field[@id="mask_dest"]'   --key name=mask_${DSTGRID}
-n 'context[@id="interpol_run"]/file_definition/file[@id="file_dst"]/field[@id="area_dest"]'   --key name=area_grid_${DSTGRID}
EOF
	;;
    esac

    cat << EOF >> command_file_${OutFileName}.txt
-n 'context[@id="interpol_read"]/domain_definition/domain[@id="domain_dst"]'                     --key type=${dstDomainType}
-n 'context[@id="interpol_read"]/domain_definition/domain[@id="domain_src"]'                     --key type=${srcDomainType}
-n 'context[@id="interpol_run"]/domain_definition/domain[@id="domain_src"]'                      --key type=${srcDomainType}
-n 'context[@id="interpol_run"]/domain_definition/domain[@id="domain_dst"]'                      --key type=${dstDomainType}
-n 'context[@id="interpol_run"]/domain_definition/domain[@id="domain_dst"]/interpolate_domain'   --key weight_filename=rmp_${OutFileName}.nc
-n 'context[@id="interpol_run"]/domain_definition/domain[@id="domain_dst"]/interpolate_domain'   --key order=${numOrder}
-n 'context[@id="interpol_run"]/domain_definition/domain[@id="domain_dst"]/interpolate_domain'   --key quantity=${Quantity}
-n 'context[@id="interpol_run"]/domain_definition/domain[@id="domain_dst"]/interpolate_domain'   --key renormalize=${Renormalize}
-n 'context[@id="interpol_run"]/domain_definition/domain[@id="domain_dst"]/interpolate_domain'   --key use_area=${useArea}
-n 'context[@id="interpol_run"]/file_definition/file[@id="dia"]'                                 --key name=dia_${OutFileName}
-n 'context[@id="interpol_run"]/file_definition/file[@id="dia"]/variable[@name="title"]'         --text "${SRC} mask interpolated to ${DST}"
-n 'context[@id="interpol_run"]/file_definition/file[@id="dia"]/variable[@name="dest_grid"]'     --text ${dstDomainType}
-n 'context[@id="interpol_run"]/file_definition/file[@id="dia"]/variable[@name="source_grid"]'   --text ${srcDomainType}
-n 'context[@id="interpol_run"]/file_definition/file[@id="dia"]/variable[@name="normalization"]' --text ${Renormalize}
-n 'context[@id="interpol_run"]/file_definition/file[@id="dia"]/variable[@name="order"]'         --text ${numOrder}   
-n 'context[@id="interpol_run"]/file_definition/file[@id="dia"]/variable[@name="quantity"]'      --text ${Quantity}
-n 'context[@id="interpol_run"]/file_definition/file[@id="dia"]/variable[@name="use_area"]'      --text ${useArea}
-n 'context[@id="interpol_run"]/file_definition/file[@id="dia"]/variable[@name="Model"]'         --text "${ModelName}"
EOF

    python3 update_xml.py -i iodef.xml -c command_file_${OutFileName}.txt
    cp iodef.xml iodef_${OutFileName}.xml
    
    ${MpiRun} ./interpol.exe --mask_src=${maskSrc} --mask_dst=${maskDst} --use_area=${useArea}

    case ${numOrder} in
	( 1 ) Text="Conservative Remapping - 1st order" ;;
	( 2 ) Text="Conservative Remapping - 2nd order" ;;
    esac
    for File in rmp_${OutFileName}.nc   dia_${OutFileName}.nc ; do
	ncatted --history \
		--attribute map_method,global,o,c,"${Text}" \
		--attribute normalization,global,o,c,"${Renormalize}" \
		--attribute Quantity,global,o,c,"${Quantity}" \
		--attribute UseArea,global,o,c,"${useArea}" ${File}
    done
done

##
echo ${Titre}"Add some metadata in file headers"${Norm}
## ===========================================================================

NCO="$(ncks --version |& tail -1 | sed 's/ncks //')"
PYTHON_VER=$( python3 -c "import sys ; print (sys.version.split(' ')[0])" )
for InFile in $(ls *${oce}_to_*${atm}_*.nc *${atm}_to_*${oce}_*.nc ${ATM}_grid_maskFrom_${OCE}.nc 2> /dev/null) ; do
    ncatted --history \
	    --attribute LongName,global,d,,                                       \
	    --attribute nco_openmp_thread_number,global,d,,                       \
	    --attribute Conventions,global,o,c,"CF-1.6"                           \
	    --attribute source,global,o,c,"IPSL Earth system model"               \
	    --attribute model,global,o,c,"${ModelName}"                           \
	    --attribute group,global,o,c,"ICMC IPSL Climate Modelling Center"     \
	    --attribute Institution,global,o,c,"IPSL https://www.ipsl.fr"         \
	    --attribute Ocean,global,o,c,"${OCE} https://www.nemo-ocean.eu"       \
	    --attribute Atmosphere,global,o,c,"${ATM} http://lmdz.lmd.jussieu.fr" \
	    --attribute production,global,o,c,"$(finger ${LOGNAME} | head -1 | awk '{print $4}')" \
	    --attribute originalFiles,global,o,c,"${OCE}_coordinates_mask.nc ${ATM}_grid_mask.nc"      \
	    --attribute associatedFiles,global,o,c,"grids_${CplModel}.nc areas_${CplModel}.nc masks_${CplModel}.nc" \
	    --attribute directory,global,o,c,"$(pwd)"                             \
	    --attribute description,global,o,c,"Fields needed by OASIS-MCT"       \
	    --attribute title,global,o,c,"${InFile}.nc"                           \
	    --attribute Program,global,o,c,"Generated by CreateWeightsMask"       \
	    --attribute timeStamp,global,o,c,"$(date)"                            \
	    --attribute HOSTNAME,global,o,c,"$(hostname)"                         \
	    --attribute LOGNAME,global,o,c,"$(whoami)"                            \
	    --attribute NCO,global,o,c,"NCO netCDF Operator ${NCO} http://nco.sourceforge.net" \
	    --attribute Python,global,o,c,"Python3 version ${PYTHON_VER}"         \
	    --attribute machine,global,o,c,"$(uname -a)"                          \
	    --attribute directory,global,o,c,"$(pwd)"                             \
	    --attribute description,global,o,c,"Generated with XIOS http://forge.ipsl.jussieu.fr/ioserver and MOSAIX https://forge.ipsl.jussieu.fr/igcmg/browser/TOOLS/MOSAIX" \
	    --attribute	SVN_Author,global,o,c,'$Author$'                 \
            --attribute SVN_Date,global,o,c,'$Date$' \
	    --attribute SVN_Revision,global,o,c,'$Revision$'               \
	    --attribute SVN_Id,global,o,c,'$Id$' \
	    ${InFile}
done

# Duplicate with no global attribute to avoid erasing attributes in other files
cp ${ATM}_grid_maskFrom_${OCE}.nc ${ATM}_grid_maskFrom_${OCE}_noglobal.nc
ncatted --history --attribute,global,d,c, ${ATM}_grid_maskFrom_${OCE}_noglobal.nc

##
echo ${Titre}"Update and complete weight files to fit OASIS requested format"${Norm}
## ===========================================================================
cat <<EOF > add_dim.nco
defdim("num_wgts",1) ;
weight[n_weight, num_wgts] = weight ;
EOF

for rmpFile in rmp_*.nc ; do
    mv ${rmpFile} tmp_${rmpFile}
    ncap2 --history --script-file add_dim.nco tmp_${rmpFile} ${rmpFile} ; rm tmp_${rmpFile}
    
    ncrename --history \
	     --dimension n_weight,num_links   \
	     --variable  src_idx,src_address  \
	     --variable  dst_idx,dst_address  \
	     --variable  weight,remap_matrix  ${rmpFile}
  
    ncatted --history --attribute conventions,global,o,c,"SCRIP" --attribute normalization,global,o,c,"none" ${rmpFile}
    
    case ${rmpFile} in
	( rmp_*${oce}_to_t${atm}_* )
	ncatted --history \
		--attribute title,global,o,c,"Weights ${OCE} to ${ATM}" \
		--attribute source_grid,global,o,c,"${oceDomainType}" \
		--attribute dest_grid,global,o,c,"${atmDomainType}"   \
		${rmpFile}	
	;;
	( rmp_*${atm}_to_*${oce}_* )
	ncatted --history \
		--attribute title,global,o,c,"Weights ${ATM} to ${OCE}" \
		--attribute source_grid,global,o,c,"${atmDomainType}" \
		--attribute dest_grid,global,o,c,"${oceDomainType}"   \
		${rmpFile}
	;;
    esac
done

##
echo ${Titre}"Add missing variables in rmp files"${Norm}
## ===========================================================================
for rmpFile in $(ls rmp_?${atm}_to_[tuv]${oce}_*.nc rmp_[tuv]${oce}_to_t${atm}_*.nc 2> /dev/null ) ; do
    echo ${rmpFile}
    a_to_o=false ; o_to_a=false
    case ${rmpFile} in
	(  rmp_?${oce}_to_?${atm}_*.nc ) o_to_a=true ;;
	(  rmp_?${atm}_to_?${oce}_*.nc ) a_to_o=true ;;
    esac

    for Grid in t u v o c ; do # Identify grids
	[[ ${rmpFile} = rmp_${Grid}${oce}_to_?${atm}_*.nc ]] && ogrid=${Grid}
	[[ ${rmpFile} = rmp_?${oce}_to_${Grid}${atm}_*.nc ]] && agrid=${Grid}
	[[ ${rmpFile} = rmp_${Grid}${atm}_to_?${oce}_*.nc ]] && agrid=${Grid}
	[[ ${rmpFile} = rmp_?${atm}_to_${Grid}${oce}_*.nc ]] && ogrid=${Grid}
    done
    OGRID=${ogrid^}
    AGRID=${agrid^}
	
    cat <<EOF >add_varoce.nco
defdim ("src_grid_size"   , \$x_grid_${OGRID}.size*\$y_grid_${OGRID}.size) ;
defdim ("src_grid_corners", 4) ;
defdim ("src_grid_rank"   , 2) ;
//
src_grid_dims[src_grid_rank] = { \$y_grid_${OGRID}.size, \$x_grid_${OGRID}.size } ;
//
src_grid_center_lat [src_grid_size] =  0.0d ;
src_grid_center_lon [src_grid_size] =  0.0d ;
src_grid_center_lat (:) = nav_lat_grid_${OGRID}(:,:)   ;
src_grid_center_lon (:) = nav_lon_grid_${OGRID}(:,:)   ;
//
src_grid_corner_lat [src_grid_size, src_grid_corners] = 0.0d ;
src_grid_corner_lon [src_grid_size, src_grid_corners] = 0.0d ;
src_grid_corner_lat(:,:) = bounds_lat_grid_${OGRID}(:,:,:) ;
src_grid_corner_lon(:,:) = bounds_lon_grid_${OGRID}(:,:,:) ;
//
src_grid_imask [src_grid_size] =    0 ;
src_grid_area  [src_grid_size] = 0.0d ;
src_grid_frac  [src_grid_size] = 1.0d ;
src_grid_imask (:) = 1 - mask_${OGRID}(:,:) ;
src_grid_imask.int() ;
src_grid_area  (:) = area_grid_${OGRID}(:,:) ;
EOF

    cp add_varoce.nco add_varoce_$(basename ${rmpFile} .nc)_o_to_a.nco
    [[ ${o_to_a} = true ]] && ncap2 --history --append --script-file add_varoce_$(basename ${rmpFile} .nc)_o_to_a.nco ${OCE}_coordinates_mask.nc ${rmpFile}
    sed -i~ -e "s/src_/dst_/g" add_varoce.nco
    cp add_varoce.nco add_varoce_$(basename ${rmpFile} .nc)_a_to_o.nco
    [[ ${a_to_o} = true ]] && ncap2 --history --append --script-file add_varoce_$(basename ${rmpFile} .nc)_a_to_o.nco ${OCE}_coordinates_mask.nc ${rmpFile}
		
    if [[ ${atm} = ico ]] ; then
	cat <<EOF >add_varatm.nco
defdim ("dst_grid_size"   , \$cell.size) ;
defdim ("dst_grid_corners", 6) ;
defdim ("dst_grid_rank"   , 2) ;
//
dst_grid_dims[dst_grid_rank] = { \$cell.size, 1 } ;
//
dst_grid_center_lat [dst_grid_size] =  0.0d ;
dst_grid_center_lon [dst_grid_size] =  0.0d ;
dst_grid_center_lat (:) = lat(:)   ;
dst_grid_center_lon (:) = lon(:)   ;
//
dst_grid_corner_lat [dst_grid_size, dst_grid_corners] = 0.0d ;
dst_grid_corner_lon [dst_grid_size, dst_grid_corners] = 0.0d ;
dst_grid_corner_lat(:,:) = bounds_lat(:,:) ;
dst_grid_corner_lon(:,:) = bounds_lon(:,:) ;
//
dst_grid_imask [dst_grid_size] =    0 ;
dst_grid_area  [dst_grid_size] = 0.0d ;
dst_grid_frac  [dst_grid_size] = 1.0d ;
dst_grid_imask (:) = 1 - Oce2AtmMask(:) ;
dst_grid_imask.int() ;
dst_grid_area  (:) = aire(:) ;
dst_grid_frac  (:) = OceFrac(:) ;
EOF
	cp add_varatm.nco add_varatm_$(basename ${rmpFile} .nc)_o_to_a.nco
	if [[ ${o_to_a} = true ]] ; then
	    ncap2 --history --append --script-file add_varatm_$(basename ${rmpFile} .nc)_o_to_a.nco ${ATM}_grid_maskFrom_${OCE}_noglobal.nc ${rmpFile}
	    ncks --alphabetize --history --overwrite --variable src_address,dst_address,remap_matrix,src_grid_dims,src_grid_center_lat,src_grid_center_lon,src_grid_corner_lon,src_grid_corner_lat,src_grid_area,src_grid_imask,dst_grid_dims,dst_grid_center_lat,dst_grid_center_lon,dst_grid_corner_lon,dst_grid_corner_lat,dst_grid_area,dst_grid_imask ${rmpFile} rmp_tmp.nc
	    mv rmp_tmp.nc ${rmpFile}
	fi
		    
	sed -i~ -e "s/dst_/src_/g" add_varatm.nco
	cp add_varatm.nco add_varatm_$(basename ${rmpFile} .nc)_a_to_o.nco
	if [[ ${a_to_o} = true ]] ; then
	    ncap2 --history --append --script-file add_varatm_$(basename ${rmpFile} .nc)_a_to_o.nco ${ATM}_grid_maskFrom_${OCE}_noglobal.nc ${rmpFile}
	    ncks --alphabetize --history --overwrite --variable src_address,dst_address,remap_matrix,src_grid_dims,src_grid_center_lat,src_grid_center_lon,src_grid_corner_lon,src_grid_corner_lat,src_grid_area,src_grid_imask,dst_grid_dims,dst_grid_center_lat,dst_grid_center_lon,dst_grid_corner_lon,dst_grid_corner_lat,dst_grid_area,dst_grid_imask ${rmpFile} rmp_tmp.nc
	    mv rmp_tmp.nc ${rmpFile}
	fi
    fi
    
    if [[ ${atm} = lmd ]] ; then
	cat <<EOF >add_varatm.nco
defdim ("dst_grid_size"   , \$lon.size*\$lat.size) ;
defdim ("dst_grid_corners", 4) ;
defdim ("dst_grid_rank"   , 2) ;
//
dst_grid_dims[dst_grid_rank] = { \$lat.size, \$lon.size } ;
//
dst_grid_center_lat [dst_grid_size] =  0.0d ;
dst_grid_center_lon [dst_grid_size] =  0.0d ;
lat0lon[lat,lon] = lat(:)+0*lon(:) ; 
lon0lat[lat,lon] = lon(:)+0*lat(:) ;
dst_grid_center_lat (:) = lat0lon(:,:)   ;
dst_grid_center_lon (:) = lon0lat(:,:)   ;
//
//dst_grid_corner_lat [dst_grid_size, dst_grid_corners] = 0.0d ; // Not available for LMDZ lon/lat grid
//dst_grid_corner_lon [dst_grid_size, dst_grid_corners] = 0.0d ;
//dst_grid_corner_lat(:,:) = bounds_lat(:,:) ;
//dst_grid_corner_lon(:,:) = bounds_lon(:,:) ;
//
dst_grid_imask [dst_grid_size] =    0 ;
dst_grid_area  [dst_grid_size] = 0.0d ;
dst_grid_frac  [dst_grid_size] = 1.0d ;
dst_grid_imask (:) = 1 - Oce2AtmMask(:,:) ;
dst_grid_imask.int() ;
dst_grid_area  (:) = aire(:,:) ;
dst_grid_frac (:)  = OceFrac(:,:) ;
EOF
	cp add_varatm.nco add_varatm_$(basename ${rmpFile} .nc)_o_to_a.nco
	if [[ ${o_to_a} = true ]] ; then
	    ncap2 --history --append --script-file add_varatm_$(basename ${rmpFile} .nc)_o_to_a.nco ${ATM}_grid_maskFrom_${OCE}_noglobal.nc ${rmpFile}
	    ncks --alphabetize --history --overwrite --variable src_address,dst_address,remap_matrix,src_grid_dims,src_grid_center_lat,src_grid_center_lon,src_grid_corner_lon,src_grid_corner_lat,src_grid_area,src_grid_imask,dst_grid_dims,dst_grid_center_lat,dst_grid_center_lon,dst_grid_area,dst_grid_imask ${rmpFile} rmp_tmp.nc
	    mv rmp_tmp.nc ${rmpFile}
	fi
	
	sed -i~ -e "s/dst/src/g" add_varatm.nco
	cp add_varatm.nco add_varatm_$(basename ${rmpFile} .nc)_a_to_o.nco
	if [[ ${a_to_o} = true ]] ; then
	    ncap2 --history --append --script-file add_varatm_$(basename ${rmpFile} .nc)_a_to_o.nco ${ATM}_grid_maskFrom_${OCE}_noglobal.nc ${rmpFile}
	    ncks --alphabetize --history --overwrite --variable src_address,dst_address,remap_matrix,src_grid_dims,src_grid_center_lat,src_grid_center_lon,src_grid_area,src_grid_imask,dst_grid_dims,dst_grid_center_lat,dst_grid_center_lon,dst_grid_corner_lon,dst_grid_corner_lat,dst_grid_area,dst_grid_imask ${rmpFile} rmp_tmp.nc
	    mv rmp_tmp.nc ${rmpFile}
	fi
    fi
done
ls -al ${OCE}_coordinates_mask.nc
##
## ============================================================================
echo ${Titre}"Creates and save auxiliary files for OASIS : grids.nc, areas.nc and masks.nc"${Norm}
cp ${SUBMIT_DIR}/CreateOasisGrids.bash .
bash CreateOasisGrids.bash --oce ${OCE} --atm ${ATM}


##
echo ${Titre}"Runoff weights"${Norm}
## ===========================================================================
if [[ "X${okRunoff}" = "Xyes" ]] ; then
    ${PyRun} python3 -u RunoffWeights.py --oce=${OCE} --atm=${ATM} \
	      --atmCoastWidth=${atmCoastWidth} --oceCoastWidth=${oceCoastWidth} --searchRadius=${searchRadius} \
	      --grids=grids_${CplModel}.nc --areas=areas_${CplModel}.nc --masks=masks_${CplModel}.nc \
	      --o2a=${ATM}_grid_maskFrom_${OCE}.nc --output=rmp_t${atm}_to_t${oce}_runoff_${runOff_atmQuantity}_to_${runOff_oceQuantity}.nc \
              --fmt=${FMT_XIOS} \
	      --atmQuantity=${runOff_atmQuantity} --oceQuantity=${runOff_oceQuantity} --ocePerio=${OcePerio}
fi

##
echo ${Titre}"Calving weights"${Norm}
## ===========================================================================
if [[ "X${okCalving}" = "Xyes" ]] ; then
    case ${OCE} in
	( eORCA025* )
	cp /ccc/work/cont003/gencmip6/deshayej/eORCA_R025_runoff_v1.2.nc .
	${PyRun} python3 -u CalvingWeights.py --output=rmp_t${atm}_to_t${oce}_calving_nosouth.nc  --fmt=${FMT_XIOS} \
		 --oce=${OCE} --atm=${ATM} --type=nosouth   \
		 --grids=grids_${CplModel}.nc --areas=areas_${CplModel}.nc --masks=masks_${CplModel}.nc \
	         --o2a=${ATM}_grid_maskFrom_${OCE}.nc
	${PyRun} python3 -u CalvingWeights.py --output=rmp_t${atm}_to_t${oce}_calving_iceberg.nc  --fmt=${FMT_XIOS} \
		 --oce=${OCE} --atm=${ATM} --type=iceberg  --repartition_file=eORCA_R025_runoff_v1.2.nc --repartition_var=Icb_flux \
		 --grids=grids_${CplModel}.nc --areas=areas_${CplModel}.nc --masks=masks_${CplModel}.nc \
	         --o2a=${ATM}_grid_maskFrom_${OCE}.nc
	${PyRun} python3 -u CalvingWeights.py --output=rmp_t${atm}_to_t${oce}_calving_iceshelf.nc --fmt=${FMT_XIOS} \
		 --oce=${OCE} --atm=${ATM} --type=iceshelf --repartition_file=eORCA_R025_runoff_v1.2.nc --repartition_var=sornfisf \
		 --grids=grids_${CplModel}.nc --areas=areas_${CplModel}.nc --masks=masks_${CplModel}.nc \
	         --o2a=${ATM}_grid_maskFrom_${OCE}.nc
	;;
	
	( eORCA1.2 | eORCA1.4 )
	cp ${R_IN}/OCE/NEMO/${OCE}/OPA/runoff-icb_DaiTrenberth_Depoorter_eORCA1_JD.nc .
	
	${PyRun} python3 -u CalvingWeights.py --output=rmp_t${atm}_to_t${oce}_calving_nosouth.nc  --fmt=${FMT_XIOS} \
		 --oce=${OCE} --atm=${ATM} --type=nosouth   \
		 --grids=grids_${CplModel}.nc --areas=areas_${CplModel}.nc --masks=masks_${CplModel}.nc \
	         --o2a=${ATM}_grid_maskFrom_${OCE}.nc
	${PyRun} python3 -u CalvingWeights.py --output=rmp_t${atm}_to_t${oce}_calving_iceberg.nc  --fmt=${FMT_XIOS} \
		 --oce=${OCE} --atm=${ATM} --type=iceberg --repartition_file=runoff-icb_DaiTrenberth_Depoorter_eORCA1_JD.nc --repartition_var=Icb_flux \
		 --grids=grids_${CplModel}.nc --areas=areas_${CplModel}.nc --masks=masks_${CplModel}.nc \
	         --o2a=${ATM}_grid_maskFrom_${OCE}.nc
	${PyRun} python3 -u CalvingWeights.py --output=rmp_t${atm}_to_t${oce}_calving_iceshelf.nc --fmt=${FMT_XIOS} \
		 --oce=${OCE} --atm=${ATM} --type=iceshelf --repartition_file=runoff-icb_DaiTrenberth_Depoorter_eORCA1_JD.nc --repartition_var=sornfisf \
		 --grids=grids_${CplModel}.nc --areas=areas_${CplModel}.nc --masks=masks_${CplModel}.nc \
	         --o2a=${ATM}_grid_maskFrom_${OCE}.nc
	;;
	
	( * )
	${PyRun} python3 -u CalvingWeights.py --output=rmp_t${atm}_to_t${oce}_calving_full.nc --fmt=${FMT_XIOS} \
		 --oce=${OCE} --atm=${ATM} --type=full --ocePerio=${OcePerio} \
		 --grids=grids_${CplModel}.nc --areas=areas_${CplModel}.nc --masks=masks_${CplModel}.nc \
	         --o2a=${ATM}_grid_maskFrom_${OCE}.nc
	;;
    esac
fi

##
echo ${Titre}"Simplifies headers, add version and comment"${Norm}
## ===========================================================================
for File in $(ls dia_*.nc rmp_*.nc ${ATM}_grid_maskFrom_${OCE}*.nc areas_${OCE}x${ATM}*.nc grids_${OCE}x${ATM}*.nc masks_${OCE}x${ATM}*.nc 2> /dev/null ) ; do
    ncatted --history --attribute history_of_appended_files,global,d,c,"" ${File}
    ncatted --history --attribute history,global,d,c,"" ${File}
    [[ "X${Comment}" != "X" ]] && ncatted --history --attribute Comment,global,o,c,"${Comment}"  ${File}
    [[ "X${Version}" != "X" ]] && ncatted --history --attribute Version,global,o,c,"${Version}"  ${File}
    [[ "X${Tag}" != "X"     ]] && ncatted --history --attribute Version,global,o,c,"${Tag}"      ${File}
done

##
echo ${Titre}"Rename rmp and dia files"${Norm}
## ===========================================================================
for File in $(ls *${oce}*${atm}* *${atm}*${oce}*  2> /dev/null ); do
    NewFile=$(echo ${File} | sed -e "s/${atm}/${ATM}/" -e "s/${oce}/${OCE}/" )
    mv ${File} ${NewFile}
done

##
echo ${Titre}"Add Tag"${Norm}
## ===========================================================================
if [[ "X${Tag}" != "X" ]] ; then
    for File in $(ls dia_*.nc rmp_*.nc ${ATM}_grid_maskFrom_${OCE}*.nc areas_${OCE}x${ATM}*.nc grids_${OCE}x${ATM}*.nc masks_${OCE}x${ATM}*.nc 2> /dev/null ) ; do
	mv ${File} $(basename ${File} .nc)_${Tag}.nc
    done
fi
##
echo ${Titre}"Add Version"${Norm}
## ===========================================================================
if [[ "X${Version}" != "X" ]] ; then
    for File in $(ls dia_*.nc rmp_*.nc ${ATM}_grid_maskFrom_${OCE}*.nc areas_${OCE}x${ATM}*.nc grids_${OCE}x${ATM}*.nc masks_${OCE}x${ATM}*.nc 2> /dev/null ) ; do
	mv ${File} $(basename ${File} .nc)_${Version}.nc
    done
fi
    
## ===========================================================================
echo ${Titre}"Save results"${Norm}
## ===========================================================================
#for File in $(ls dia_*.nc rmp_*.nc ${ATM}_grid_maskFrom_${OCE}*.nc areas_${OCE}x${ATM}*.nc grids_${OCE}x${ATM}*.nc masks_${OCE}x${ATM}*.nc 2> /dev/null ) ; do
#    cp ${File} ${SUBMIT_DIR}
#done


##
echo ${Titre}"Creates a README.txt file"${Norm}
## ===========================================================================
[[ -f README.txt ]] && rm README.txt
UUID=$(uuid)

cat <<EOF > README.txt
Files produced by CreateWeightsMask.bash and CreateOasisGrids.bash

rmp_* are weights files
dia_* are diagnostic files not needed for the coupler
grids_${CplModel}.nc areas_${CplModel}.nc masks_${CplModel}.nc are auxiliary file needed by OASIS-MCT
All files have the same uuid in the global attributes

Description      : Weigths and auxiliary files for coupling ${OCE} and ${ATM} needed by OASIS-MCT
Conventions      : CF-1.6
Source           : IPSL Earth system model
Model            : ${ModelName}
Group            : ICMC IPSL Climate Modelling Center
Institution      : IPSL https://www.ipsl.fr
Ocean            : ${OCE} https://www.nemo-ocean.eu
Atmosphere       : ${ATM} http://lmdz.lmd.jussieu.fr
Production       : $(finger ${LOGNAME} | head -1 | awk '{print $4}') 
Original files   : ${OCE}_coordinates_mask.nc ${ATM}_grid_mask.nc
Associated files : grids_${CplModel}.nc areas_${CplModel}.nc masks_${CplModel}.nc
Description      : Generated with XIOS http://forge.ipsl.jussieu.fr/ioserver and MOSAIX https://forge.ipsl.jussieu.fr/igcmg/browser/TOOLS/MOSAIX"
directory        : $(pwd)
timeStamp        : $(date)
Machine info     : $(uname -a)
uuid             : ${UUID}
HOSTNAME         : $(hostname)
LOGNAME          : $(whoami)
NCO              : NCO netCDF Operator ${NCO} http://nco.sourceforge.net
Python version   : ${PYTHON_VER}

EOF

echo 'SVN Information : '                                              >> README.txt
echo '$Author$ '                                              >> README.txt
echo '$Date$ '          >> README.txt
echo '$Revision$ '                                              >> README.txt
echo '$Id$ ' >> README.txt
echo '$HeadURL$ ' >> README.txt

echo ${Titre}"Compute checksums and add them to README"${Norm}
# ------------------------------------------------------------
cat << EOF >> README.txt

Files produced, with checksum produced by Unix command shasum (version $(shasum --version)) with default algorithm

Checksum                                  File
========================================================================================================
EOF

for file in $(ls dia_*.nc rmp_*.nc ${ATM}_grid_maskFrom_${OCE}*.nc areas_${OCE}x${ATM}*.nc grids_${OCE}x${ATM}*.nc masks_${OCE}x${ATM}*.nc 2> /dev/null ) ; do
    ncatted --history --attribute uuid,global,o,c,"${UUID}" ${file}
    echo "$(shasum ${file})" >> README.txt
done

cat <<EOF >> README.txt

================= That's all folk's ! ========================
EOF

if [[ "X${Version}" != "X" ]] ; then
    cp README.txt ${SUBMIT_DIR}/README_${CplModel}_MOSAIX_${Version}.txt
else
    cp README.txt ${SUBMIT_DIR}/README_${CplModel}_MOSAIX.txt
fi
## ===========================================================================
##
echo ${Titre}"     That's all folk's !!!    "${Norm}
##
## ===========================================================================

